@@ -45,14 +45,19 @@ namespace GPBoost {
 		* \param dim_gp_coords Dimension of the coordinates (=number of features) for Gaussian process
 		* \param gp_rand_coef_data Covariate data for Gaussian process random coefficients
 		* \param num_gp_rand_coef Number of Gaussian process random coefficients
-		* \param cov_fct_shape Shape parameter of covariance function (=smoothness parameter for Matern and Wendland covariance. For the Wendland covariance function, we follow the notation of Bevilacqua et al. (2018)). This parameter is irrelevant for some covariance functions such as the exponential or Gaussian.
-		* \param cov_fct_taper_range Range parameter of Wendland covariance function / taper. We follow the notation of Bevilacqua et al. (2018)
-		* \param vecchia_approx If true, the Veccia approximation is used for the Gaussian process
+		* \param cov_fct Type of covariance function for Gaussian process (GP). We follow the notation and parametrization of Diggle and Ribeiro (2007) except for the Matern covariance where we follow Rassmusen and Williams (2006)
+		* \param cov_fct_shape Shape parameter of covariance function (=smoothness parameter for Matern and Wendland covariance. This parameter is irrelevant for some covariance functions such as the exponential or Gaussian
+		* \param gp_approx Type of GP-approximation for handling large data
+		* \param cov_fct_taper_range Range parameter of the Wendland covariance function and Wendland correlation taper function. We follow the notation of Bevilacqua et al. (2019, AOS)
+		* \param cov_fct_taper_shape Shape parameter of the Wendland covariance function and Wendland correlation taper function. We follow the notation of Bevilacqua et al. (2019, AOS)
 		* \param num_neighbors The number of neighbors used in the Vecchia approximation
 		* \param vecchia_ordering Ordering used in the Vecchia approximation. "none" = no ordering, "random" = random ordering
 		* \param vecchia_pred_type Type of Vecchia approximation for making predictions. "order_obs_first_cond_obs_only" = observed data is ordered first and neighbors are only observed points, "order_obs_first_cond_all" = observed data is ordered first and neighbors are selected among all points (observed + predicted), "order_pred_first" = predicted data is ordered first for making predictions, "latent_order_obs_first_cond_obs_only"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are only observed points, "latent_order_obs_first_cond_all"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are selected among all points
 		* \param num_neighbors_pred The number of neighbors used in the Vecchia approximation for making predictions
-		* \param likelihood Likelihood function for the observed response variable. Default = "gaussian"
+		* \param num_ind_points Number of inducing points / knots for, e.g., a predictive process approximation
+		* \param likelihood Likelihood function for the observed response variable
+		* \param matrix_inversion_method Method which is used for matrix inversion
+		* \param seed Seed used for model creation (e.g., random ordering in Vecchia approximation)
 		*/
 		LIGHTGBM_EXPORT REModel(data_size_t num_data,
 			const data_size_t* cluster_ids_data,
@@ -69,13 +74,17 @@ namespace GPBoost {
 			data_size_t num_gp_rand_coef,
 			const char* cov_fct,
 			double cov_fct_shape,
+			const char* gp_approx,
 			double cov_fct_taper_range,
-			bool vecchia_approx,
+			double cov_fct_taper_shape,
 			int num_neighbors,
 			const char* vecchia_ordering,
 			const char* vecchia_pred_type,
 			int num_neighbors_pred,
-			const char* likelihood);
+			int num_ind_points,
+			const char* likelihood,
+			const char* matrix_inversion_method,
+			int seed);
 
 		/*! \brief Destructor */
 		LIGHTGBM_EXPORT ~REModel();
@@ -127,12 +136,14 @@ namespace GPBoost {
 		* \param lr_coef Learning rate for fixed-effect linear coefficients
 		* \param acc_rate_coef Acceleration rate for coefficients for Nesterov acceleration (only relevant if nesterov_schedule_version == 0)
 		* \param optimizer_coef Optimizer for linear regression coefficients
-		* \param matrix_inversion_method Method which is used for matrix inversion
 		* \param cg_max_num_it Maximal number of iterations for conjugate gradient algorithm
 		* \param cg_max_num_it_tridiag Maximal number of iterations for conjugate gradient algorithm when being run as Lanczos algorithm for tridiagonalization
 		* \param cg_delta_conv Tolerance level for L2 norm of residuals for checking convergence in conjugate gradient algorithm when being used for parameter estimation
 		* \param num_rand_vec_trace Number of random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix
 		* \param reuse_rand_vec_trace If true, random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix are sampled only once at the beginning and then reused in later trace approximations, otherwise they are sampled everytime a trace is calculated
+		* \param cg_preconditioner_type Type of preconditioner used for the conjugate gradient algorithm
+		* \param seed_rand_vec_trace Seed number to generate random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix
+		* \param piv_chol_rank Rank of the pivoted cholseky decomposition used as preconditioner of the conjugate gradient algorithm	
 		*/
 		void SetOptimConfig(double* init_cov_pars,
 			double lr,
@@ -151,12 +162,14 @@ namespace GPBoost {
 			double lr_coef,
 			double acc_rate_coef,
 			const char* optimizer_coef,
-			const char* matrix_inversion_method,
 			int cg_max_num_it,
 			int cg_max_num_it_tridiag,
 			double cg_delta_conv,
 			int num_rand_vec_trace,
-			bool reuse_rand_vec_trace);
+			bool reuse_rand_vec_trace,
+			const char* cg_preconditioner_type,
+			int seed_rand_vec_trace,
+			int piv_chol_rank);
 
 		/*!
 		* \brief Reset cov_pars_ (to their initial values).
@@ -169,8 +182,11 @@ namespace GPBoost {
 		*		For the GPBoost algorithm for Gaussian data, this equals F - y where F is the fitted value of the ensemble at the training data and y the response data.
 		*		For the GPBoost algorithm for non-Gaussian data, this is ignored (and can be nullptr) as the response data has been set before.
 		* \param fixed_effects Fixed effects component F of location parameter (only used for non-Gaussian data). For Gaussian data, this is ignored
+		* \param called_in_GPBoost_algorithm If true, this function is called in the GPBoost algorithm, otherwise for the estimation of a GLMM
 		*/
-		void OptimCovPar(const double* y_data, const double* fixed_effects);
+		void OptimCovPar(const double* y_data,
+			const double* fixed_effects,
+			bool called_in_GPBoost_algorithm);
 
 		/*!
 		* \brief Find covariance parameters and linear regression coefficients (if there are any) that minimize the (approximate) negative log-ligelihood
@@ -179,7 +195,9 @@ namespace GPBoost {
 		* \param covariate_data Covariate data (=independent variables, features)
 		* \param num_covariates Number of covariates
 		*/
-		void OptimLinRegrCoefCovPar(const double* y_data, const double* covariate_data, int num_covariates);
+		void OptimLinRegrCoefCovPar(const double* y_data,
+			const double* covariate_data,
+			int num_covariates);
 
 		/*!
 		* \brief Find constant initial value of ensenmble for boosting (used only for non-Gaussian data). 
@@ -361,7 +379,8 @@ namespace GPBoost {
 		* \param[out] leaf_values Leaf values when performing a Newton update step (array of size num_leaves)
 		*/
 		void NewtonUpdateLeafValues(const int* data_leaf_index,
-			const int num_leaves, double* leaf_values) const;
+			const int num_leaves,
+			double* leaf_values) const;
 
 		/*!
 		* \brief If cov_pars_ is is not defined, define them as init_cov_pars_ or if init_cov_pars_ is not given, find "reasonable" default values for the intial values of the covariance parameters
@@ -371,9 +390,10 @@ namespace GPBoost {
 
 	private:
 
-		bool sparse_ = false;
-		std::unique_ptr < REModelTemplate<sp_mat_t, chol_sp_mat_t> > re_model_sp_;
-		std::unique_ptr < REModelTemplate<den_mat_t, chol_den_mat_t> > re_model_den_;
+		string_t matrix_format_ = "den_mat_t";//den_mat_t, sp_mat_t, sp_mat_rm_t
+		std::unique_ptr<REModelTemplate<sp_mat_t, chol_sp_mat_t>> re_model_sp_;
+		std::unique_ptr<REModelTemplate<sp_mat_rm_t, chol_sp_mat_rm_t>> re_model_sp_rm_;
+		std::unique_ptr<REModelTemplate<den_mat_t, chol_den_mat_t>> re_model_den_;
 		vec_t cov_pars_; //Covariance paramters
 		vec_t init_cov_pars_; //Initial values for covariance parameters
 		bool cov_pars_initialized_ = false; //This is true if InitializeCovParsIfNotDefined() has been called
