@@ -1333,15 +1333,19 @@ GPBOOST_C_EXPORT int LGBM_NetworkInitWithFunctions(int num_machines,
 * \param dim_gp_coords Dimension of the coordinates (=number of features) for Gaussian process
 * \param gp_rand_coef_data Covariate data for Gaussian process random coefficients
 * \param num_gp_rand_coef Number of Gaussian process random coefficients
-* \param cov_fct Type of covariance (kernel) function for Gaussian processes. We follow the notation and parametrization of Diggle and Ribeiro (2007) except for the Matern covariance where we follow Rassmusen and Williams (2006)
-* \param cov_fct_shape Shape parameter of covariance function (=smoothness parameter for Matern and Wendland covariance. For the Wendland covariance function, we follow the notation of Bevilacqua et al. (2018)). This parameter is irrelevant for some covariance functions such as the exponential or Gaussian.
-* \param cov_fct_taper_range Range parameter of Wendland covariance function / taper. We follow the notation of Bevilacqua et al. (2018)
-* \param vecchia_approx If true, the Veccia approximation is used for the Gaussian process
+* \param cov_fct Type of covariance function for Gaussian process (GP). We follow the notation and parametrization of Diggle and Ribeiro (2007) except for the Matern covariance where we follow Rassmusen and Williams (2006)
+* \param cov_fct_shape Shape parameter of covariance function (=smoothness parameter for Matern and Wendland covariance. This parameter is irrelevant for some covariance functions such as the exponential or Gaussian
+* \param gp_approx Type of GP-approximation for handling large data
+* \param cov_fct_taper_range Range parameter of the Wendland covariance function and Wendland correlation taper function. We follow the notation of Bevilacqua et al. (2019, AOS)
+* \param cov_fct_taper_shape Shape parameter of the Wendland covariance function and Wendland correlation taper function. We follow the notation of Bevilacqua et al. (2019, AOS)
 * \param num_neighbors The number of neighbors used in the Vecchia approximation
 * \param vecchia_ordering Ordering used in the Vecchia approximation. "none" = no ordering, "random" = random ordering
 * \param vecchia_pred_type Type of Vecchia approximation for making predictions. "order_obs_first_cond_obs_only" = observed data is ordered first and neighbors are only observed points, "order_obs_first_cond_all" = observed data is ordered first and neighbors are selected among all points (observed + predicted), "order_pred_first" = predicted data is ordered first for making predictions, "latent_order_obs_first_cond_obs_only"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are only observed points, "latent_order_obs_first_cond_all"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are selected among all points
 * \param num_neighbors_pred The number of neighbors used in the Vecchia approximation for making predictions
-* \param likelihood Likelihood function for the observed response variable. Default = "gaussian"
+* \param num_ind_points Number of inducing points / knots for, e.g., a predictive process approximation
+* \param likelihood Likelihood function for the observed response variable
+* \param matrix_inversion_method Method which is used for matrix inversion
+* \param seed Seed used for model creation (e.g., random ordering in Vecchia approximation)
 * \param[out] out Created REModel
 * \return 0 when succeed, -1 when failure happens
 */
@@ -1360,13 +1364,17 @@ GPBOOST_C_EXPORT int GPB_CreateREModel(int32_t num_data,
     int32_t num_gp_rand_coef,
     const char* cov_fct,
     double cov_fct_shape,
+    const char* gp_approx,
     double cov_fct_taper_range,
-    bool vecchia_approx,
+    double cov_fct_taper_shape,
     int num_neighbors,
     const char* vecchia_ordering,
     const char* vecchia_pred_type,
     int num_neighbors_pred,
+    int num_ind_points,
     const char* likelihood,
+    const char* matrix_inversion_method,
+    int seed,
     REModelHandle* out);
 
 /*!
@@ -1396,12 +1404,14 @@ GPBOOST_C_EXPORT int GPB_REModelFree(REModelHandle handle);
 * \param lr_coef Learning rate for fixed-effect linear coefficients
 * \param acc_rate_coef Acceleration rate for coefficients for Nesterov acceleration (only relevant if nesterov_schedule_version == 0)
 * \param optimizer_coef Optimizer for linear regression coefficients
-* \param matrix_inversion_method Method which is used for matrix inversion
 * \param cg_max_num_it Maximal number of iterations for conjugate gradient algorithm
 * \param cg_max_num_it_tridiag Maximal number of iterations for conjugate gradient algorithm when being run as Lanczos algorithm for tridiagonalization
 * \param cg_delta_conv Tolerance level for L2 norm of residuals for checking convergence in conjugate gradient algorithm when being used for parameter estimation
 * \param num_rand_vec_trace Number of random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix
 * \param reuse_rand_vec_trace If true, random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix are sampled only once at the beginning and then reused in later trace approximations, otherwise they are sampled everytime a trace is calculated
+* \param cg_preconditioner_type Type of preconditioner used for the conjugate gradient algorithm
+* \param seed_rand_vec_trace Seed number to generate random vectors (e.g. Rademacher) for stochastic approximation of the trace of a matrix
+* \param piv_chol_rank Rank of the pivoted cholseky decomposition used as preconditioner of the conjugate gradient algorithm
 * \return 0 when succeed, -1 when failure happens
 */
 GPBOOST_C_EXPORT int GPB_SetOptimConfig(REModelHandle handle,
@@ -1422,12 +1432,14 @@ GPBOOST_C_EXPORT int GPB_SetOptimConfig(REModelHandle handle,
     double lr_coef,
     double acc_rate_coef,
     const char* optimizer_coef,
-    const char* matrix_inversion_method,
     int cg_max_num_it,
     int cg_max_num_it_tridiag,
     double cg_delta_conv,
     int num_rand_vec_trace,
-    bool reuse_rand_vec_trace);
+    bool reuse_rand_vec_trace,
+    const char* cg_preconditioner_type,
+    int seed_rand_vec_trace,
+    int piv_chol_rank);
 
 /*!
 * \brief Find parameters that minimize the negative log-ligelihood (=MLE)
